@@ -29,7 +29,7 @@ from quark.utils.output import (
     output_parent_function_json,
     output_parent_function_table,
 )
-from quark.utils.pprint import print_info, print_success
+from quark.utils.pprint import print_info, print_success, print_warning
 from quark.utils.weight import Weight
 
 MAX_SEARCH_LAYER = 3
@@ -334,7 +334,7 @@ class Quark:
         for method in potential_method_list:
             current_class_set = {method.class_name}
 
-            while not current_class_set.intersection(
+            while current_class_set and not current_class_set.intersection(
                 {class_name, "Ljava/lang/Object;"}
             ):
                 next_class_set = set()
@@ -423,6 +423,17 @@ class Quark:
                     second_api_xref_from
                 )
 
+                if not first_api_xref_from:
+                    print_warning(
+                        f"Unable to find the upperfunc of {first_api}"
+                    )
+                    continue
+                if not second_api_xref_from:
+                    print_warning(
+                        f"Unable to find the upperfunc of{second_api}"
+                    )
+                    continue
+
                 mutual_parent_function_list = self.find_intersection(
                     first_api_xref_from, second_api_xref_from
                 )
