@@ -18,10 +18,33 @@ from quark.core.axmlreader import AxmlReader
 from quark.core.interface.baseapkinfo import BaseApkinfo
 from quark.core.struct.bytecodeobject import BytecodeObject
 from quark.core.struct.methodobject import MethodObject
-from quark.utils.tools import descriptor_to_androguard_format
+from quark.utils.tools import descriptor_to_androguard_format, remove_dup_list
 
 RizinCache = namedtuple("rizin_cache", "address dexindex is_imported")
 
+PRIMITIVE_TYPE_MAPPING = {
+    "void": "V",
+    "boolean": "Z",
+    "byte": "B",
+    "char": "C",
+    "short": "S",
+    "int": "I",
+    "long": "J",
+    "float": "F",
+    "double": "D",
+    "Boolean": "Ljava/lang/Boolean;",
+    "Byte": "Ljava/lang/Byte;",
+    "Character": "Ljava/lang/Character;",
+    "Short": "Ljava/lang/Short;",
+    "Integer": "Ljava/lang/Integer;",
+    "Long": "Ljava/lang/Long;",
+    "Float": "Ljava/lang/Float;",
+    "Double": "Ljava/lang/Double;",
+    "String": "Ljava/lang/String;",
+}
+
+RIZIN_ESCAPE_CHAR_LIST = ["<", ">", "$"]
+
 
 class RizinImp(BaseApkinfo):
     def __init__(
@@ -65,6 +88,29 @@ class RizinImp(BaseApkinfo):
         rz.cmd("aa")
         return rz
 
+    def _convert_type_to_type_signature(self, raw_type: str):
+        if raw_type.endswith("[]"):
+            return "[" + self._convert_type_to_type_signature(raw_type[:-2])
+
+        if raw_type.startswith("["):
+            return "[" + self._convert_type_to_type_signature(raw_type[1:])
+
+        if raw_type in PRIMITIVE_TYPE_MAPPING:
+            return PRIMITIVE_TYPE_MAPPING[raw_type]
+
+        if "." in raw_type or "_" in raw_type:
+            raw_type = raw_type.replace(".", "/")
+            raw_type = raw_type.replace("_", "$")
+            return "L" + raw_type + ";"
+
+        return raw_type
+
+    @staticmethod
+    def _escape_str_in_rizin_manner(raw_str: str):
+        for c in RIZIN_ESCAPE_CHAR_LIST:
+            raw_str = raw_str.replace(c, "_")
+        return raw_str
+
     @functools.lru_cache
     def _get_methods_classified(self, dexindex):
         rz = self._get_rz(dexindex)
@@ -75,25 +121,108 @@ class RizinImp(BaseApkinfo):
             if json_obj.get("type") not in ["FUNC", "METH"]:
                 continue
 
-            full_name = json_obj["realname"]
-            class_name, method_descriptor = full_name.split(".method.", maxsplit=1)
-            class_name = class_name + ";"
+            # -- Descriptor --
+            full_method_name = json_obj["name"]
+            raw_argument_str = next(
+                re.finditer("\\(.*\\).*", full_method_name), None
+            )
+            if raw_argument_str is None:
+                continue
+            raw_argument_str = raw_argument_str.group(0)
 
-            delimiter = method_descriptor.index('(')
-            methodname = method_descriptor[:delimiter]
-            descriptor = method_descriptor[delimiter:]
-            descriptor = descriptor_to_androguard_format(descriptor)
+            if raw_argument_str.endswith(")"):
+                # Convert Java lauguage type to JVM type signature
 
+                # Parse the arguments
+                raw_argument_str = raw_argument_str[1:-1]
+                arguments = [
+                    self._convert_type_to_type_signature(arg)
+                    for arg in raw_argument_str.split(", ")
+                ]
+
+                # Parse the return type
+                return_type = next(
+                    re.finditer(
+                        "[A-Za-zL][A-Za-z0-9L/\\;[\\]$.]+ ", full_method_name
+                    ),
+                    None,
+                )
+                if return_type is None:
+                    print(f"Unresolved method signature: {full_method_name}")
+                    continue
+                return_type = return_type.group(0).strip()
+
+                # Convert
+                raw_argument_str = (
+                    "("
+                    + " ".join(arguments)
+                    + ")"
+                    + self._convert_type_to_type_signature(return_type)
+                )
+
+            descriptor = descriptor_to_androguard_format(raw_argument_str)
+
+            # -- Method name --
+            method_name = json_obj["realname"]
+
+            # -- Is imported --
             is_imported = json_obj["is_imported"]
 
+            # -- Class name --
+            # Test if the class name is truncated
+            escaped_method_name = self._escape_str_in_rizin_manner(method_name)
+            if escaped_method_name.endswith("_"):
+                escaped_method_name = escaped_method_name[:-1]
+
+            flag_name = json_obj["flagname"]
+
+            # sym.imp.clone doesn't belong to a class
+            if flag_name == "sym.imp.clone":
+                method = MethodObject(
+                    class_name="",
+                    name="clone",
+                    descriptor="()Ljava/lang/Object;",
+                    cache=RizinCache(json_obj["vaddr"], dexindex, is_imported),
+                )
+                method_dict[""].append(method)
+                continue
+
+            if escaped_method_name not in flag_name:
+                logging.warning(
+                    f"The class name may be truncated: {json_obj['flagname']}"
+                )
+
+            # Drop the method name
+            match = None
+            for match in re.finditer("_+[A-Za-z]+", flag_name):
+                pass
+            if match is None:
+                logging.warning(
+                    f"Skip the damaged flag: {json_obj['flagname']}"
+                )
+                continue
+            match = match.group(0)
+            flag_name = flag_name[: flag_name.rfind(match)]
+
+            # Drop the prefixes sym. and imp.
+            while flag_name.startswith("sym.") or flag_name.startswith("imp."):
+                flag_name = flag_name[4:]
+
+            class_name = self._convert_type_to_type_signature(flag_name)
+
+            # Append the method
             method = MethodObject(
                 class_name=class_name,
-                name=methodname,
+                name=method_name,
                 descriptor=descriptor,
                 cache=RizinCache(json_obj["vaddr"], dexindex, is_imported),
             )
             method_dict[class_name].append(method)
 
+        # Remove duplicates
+        for class_name, method_list in method_dict.items():
+            method_dict[class_name] = remove_dup_list(method_list)
+
         return method_dict
 
     @functools.cached_property
@@ -168,7 +297,14 @@ class RizinImp(BaseApkinfo):
                 continue
 
             if "fcn_addr" in xref:
-                upperfunc_set.add(self._get_method_by_address(xref["fcn_addr"]))
+                matched_method = self._get_method_by_address(xref["fcn_addr"])
+                if not matched_method:
+                    logging.debug(
+                        f"Cannot identify function at {xref['fcn_addr']}."
+                    )
+                    continue
+
+                upperfunc_set.add(matched_method)
             else:
                 logging.debug(
                     f"Key from was not found at searching"
@@ -194,10 +330,19 @@ class RizinImp(BaseApkinfo):
                 continue
 
             if "to" in xref:
+                matched_method = self._get_method_by_address(xref["to"])
+                if not matched_method:
+                    logging.debug(
+                        f"Cannot identify function at {xref['fcn_addr']}."
+                    )
+                    continue
+
+                offset = xref["from"] - cache.address
+
                 lowerfunc_set.add(
                     (
-                        self._get_method_by_address(xref["to"]),
-                        xref["from"] - cache.address,
+                        matched_method,
+                        offset,
                     )
                 )
             else:
@@ -310,17 +455,12 @@ class RizinImp(BaseApkinfo):
 
             rz = self._get_rz(dex_index)
 
-            hierarchy_graph = rz.cmd("icg").split("\n")
+            class_info_list = rz.cmdj("icj")
+            for class_info in class_info_list:
+                class_name = class_info["classname"]
+                super_class = class_info["super"]
 
-            for element in hierarchy_graph:
-                if element.startswith("age"):
-                    element_part = element.split()
-                    for index, class_name in enumerate(element_part):
-                        if not class_name.endswith(";"):
-                            element_part[index] = class_name + ";"
-
-                    for subclass in element_part[2:]:
-                        hierarchy_dict[subclass].add(element_part[1])
+                hierarchy_dict[class_name].add(super_class)
 
         return hierarchy_dict
 
@@ -332,16 +472,12 @@ class RizinImp(BaseApkinfo):
 
             rz = self._get_rz(dex_index)
 
-            hierarchy_graph = rz.cmd("icg").split("\n")
+            class_info_list = rz.cmdj("icj")
+            for class_info in class_info_list:
+                class_name = class_info["classname"]
+                super_class = class_info["super"]
 
-            for element in hierarchy_graph:
-                if element.startswith("age"):
-                    element_part = element.split()
-                    for index, class_name in enumerate(element_part):
-                        if not class_name.endswith(";"):
-                            element_part[index] = class_name + ";"
-
-                    hierarchy_dict[element_part[1]].update(element_part[2:])
+                hierarchy_dict[super_class].add(class_name)
 
         return hierarchy_dict
 
@@ -364,8 +500,8 @@ class RizinImp(BaseApkinfo):
         mnemonic, args = smali.split(maxsplit=1)  # Split into twe parts
 
         # invoke-kind instruction may left method index at the last
-        if mnemonic.startswith("invoke"):
-            args = args[: args.rfind(" ;")]
+        # if mnemonic.startswith("invoke"):
+        #     args = args[: args.rfind(" ;")]
 
         args = [arg.strip() for arg in re.split("[{},]+", args) if arg]
 
@@ -376,7 +512,7 @@ class RizinImp(BaseApkinfo):
             args = args[:-1]
 
             if mnemonic.startswith("invoke"):
-                parameter = re.sub(r"\.", ";->", parameter, count=1)
+                parameter = re.sub(r"\.", "->", parameter, count=1)
 
         register_list = []
         # Ranged registers
